\    Title:  Tests for dstrings utilities
\   Author:  David N. Williams
\     File:  dstring-utils-test.fs
\ Log file:  dstring-utils.log
\  License:  Public Domain
\  Version:  0.8.6
\  Revised:  August 21, 2010

[UNDEFINED] \\ [IF]
: \\  ( -- )  -1 parse 2drop BEGIN refill 0= UNTIL ; [THEN]

s" FORTH-NAME" ENVIRONMENT? dup [IF]
  drop s" pfe" compare 0=
  0 REDEFINED-MSG !
[THEN] CONSTANT PFE-HOST

s" gforth" environment? dup [IF]
  ( s flag) nip nip
  0 WARNINGS !
[THEN] CONSTANT GFORTH-HOST

s" IFORTH" environment? dup [IF]
  ( flag) drop false CONSTANT ANSI-TERMINAL-COLORS
  0 WARNING !
[THEN] CONSTANT IFORTH-HOST

\ These are reset to FALSE unless the host is pfe.  Set to TRUE
\ to load the corresponding module when pfe is the host.
true VALUE USE-DSTRINGS-EXT
false VALUE USE-FPNOSTACK-EXT

PFE-HOST 0= [IF]
  false to USE-DSTRINGS-EXT   false to USE-FPNOSTACK-EXT
[THEN]

\ Ensure that fp is enabled for pfe in ttester.
USE-FPNOSTACK-EXT [IF]
  s" FPNOSTACK-EXT" ENVIRONMENT? [IF] ( flag) drop [THEN]
[ELSE]
  s" FLOATING-EXT" ENVIRONMENT? [IF] ( flag) drop [ELSE]
  cr .( FLOATING POINT NOT AVAILABLE) cr ABORT [THEN]
[THEN]

s" ttester.fs" included  \ gforth cvs revision 1.13 known to work
true VERBOSE !
decimal

\ ***BEGIN TTESTER DISPLAY 1.0.3

\ Turn this off if it causes a problem with your terminal.
[UNDEFINED] ANSI-TERMINAL-COLORS [IF]
  true CONSTANT ANSI-TERMINAL-COLORS
[THEN]

VARIABLE #errors    0 #errors !

ANSI-TERMINAL-COLORS [IF]  \ Krishna Myneni
#27 CONSTANT esc
: normal-text ( -- )  esc emit ." [0m" ; 
: red-text    ( -- )  esc emit ." [31m" ;
: green-text  ( -- )  esc emit ." [32m" ;
: blue-text   ( -- )  esc emit ." [34m" ;  
[ELSE]
: normal-text ( -- ) ; immediate
: red-text    ( -- ) ; immediate
: green-text  ( -- ) ; immediate
: blue-text   ( -- ) ; immediate
[THEN]

:noname  ( c-addr u -- )
\ Display an error message followed by the line that had the
\ error.
  red-text 1 #errors +! error1 normal-text ; ERROR-XT !

: ?.    ( -- )    VERBOSE @ IF . THEN ;
: ?.cr  ( -- )    VERBOSE @ IF cr THEN ;
: ?.(   ( -- )    VERBOSE @ IF postpone .( ELSE postpone ( THEN ;
: ?type ( s -- )  VERBOSE @ IF type ELSE 2drop THEN ;

: ?.errors  ( -- )
  VERBOSE @
  IF blue-text ." #ERRORS: " normal-text #errors @ . THEN ;

: COMMENT-NCR ( -- )  \ Krishna Myneni
  blue-text
  source >in @ /string
  ?type
  source >in ! drop
  normal-text ;
: COMMENT ( -- )  COMMENT-NCR ?.cr ;

\ ***END TTESTER DISPLAY

\ before first displayed line
PFE-HOST IFORTH-HOST or [IF] ?.cr [THEN]

PFE-HOST [IF]
  COMMENT Test host is pfe.
[THEN]
GFORTH-HOST [IF]
  COMMENT Test host is gforth.
[THEN]

true CONSTANT USE-FP             \ turn on fp words in dstring-utils
#128 CONSTANT TRY-MAX-FNUM-DIGS  \ less than default
TRY-MAX-FNUM-DIGS CONSTANT MAX-FNUM-DIGS
s" dstring-utils.fs" included
decimal

USE-DSTRINGS-EXT [IF]
  COMMENT Using pfe dstrings module.
[ELSE]
  COMMENT Using dstrings.fs.
[THEN]

:noname  ( -- fpstack? )  depth >r 1.1E0 depth >r fdrop 2r> <> ;
EXECUTE [IF]
  COMMENT Nonseparate floating point stack.
[ELSE]
  COMMENT Separate floating point stack.
[THEN]

: s=      ( s1 s2 -- flag )  compare 0= ;
: cat$>s   ( -- s )  ENDCAT $s> ;
: cat=    ( s -- flag ) cat$>s s= ;

s" MAX-U" ENVIRONMENT? 0=
[IF] cr .( ***MAX-U query undefined) \\ [THEN]
CONSTANT MAX-U

s" MAX-N" ENVIRONMENT? 0=
[IF] cr .( ***MAX-N query undefined) \\ [THEN]
CONSTANT MAX-N
MAX-N negate CONSTANT -MAX-N  \ assuming 2's complement?

TESTING  NUM+ UNUM+ UNUM>HEX+ UNUM0R+ -MON-

\ The iteration test for each cat word checks cat purity.

\ num+  ( n -- )
t{ 0 num+ s" 0" cat= -> true }t
t{ $+" ccc" -5000 num+ s" ccc-5000" cat= -> true }t
t{ MAX-N num+ cat$>s evaluate -> MAX-N }t
t{ $+" ccc" -5000 hex num+ decimal s" ccc-1388" cat= -> true }t
t{ 5 num+ 6 num+ s" 56" cat= -> true }t

\ unum+  ( u -- )
t{ 0 unum+ s" 0" cat= -> true }t
t{ $+" ccc" 5000 unum+ s" ccc5000" cat= -> true }t
t{ MAX-U unum+ cat$>s evaluate -> MAX-U }t
t{ $+" ccc" 5000 hex unum+ decimal s" ccc1388" cat= -> true }t
t{ 123 unum+ 45 num+ s" 12345" cat= -> true }t

\ unum>hex+  ( u -- )
t{ 0 unum>hex+ s" 0x0" cat= -> true }t
t{ $+" ccc" 5000 unum>hex+ s" ccc0x1388" cat= -> true }t
t{ MAX-U unum>hex+ cat$>s 2 /string hex evaluate decimal -> MAX-U }t
hex
t{ 1238 unum>hex+ 0 unum>hex+ s" 0x12380x0" cat= -> true }t
decimal

\ unum0r+  ( u r --  )
t{ 0 1 unum0r+ s" 0"  cat= -> true }t
t{ 0 2 unum0r+ s" 00" cat= -> true }t
t{ $+" ccc" 5000 4 unum0r+ s" ccc5000"    cat= -> true }t
t{ $+" ccc" 5000 2 unum0r+ s" ccc00"      cat= -> true }t
t{ $+" ccc" 5000 7 unum0r+ s" ccc0005000" cat= -> true }t
t{ $+" ccc" 5000 7 hex unum0r+ decimal s" ccc0001388" cat= -> true }t
t{ 12345 3 unum0r+ 67 1 unum0r+ s" 3457" cat= -> true }t

\ -mon-  ( month -- month.s )
t{  1 -mon- s" -Jan-" s= -> true }t
t{ 12 -mon- s" -Dec-" s= -> true }t

[DEFINED] fnum+ [IF]
TESTING  FNUM+ SET-FNUM-DIGS

3 set-fnum-digs

\ fnum+   ( f: num  -- $: num$ )
PFE-HOST [IF]
t{ 0E fnum+  s" .000E0" cat= -> true }t
0 [IF] \ actually returns ".00", sprintf problem?
t{ -0E0 fnum+  s" -.000E0" cat= -> true }t
[THEN]
[THEN]  \ FNUM+

GFORTH-HOST [IF]
t{  0E  fnum+  s" .000E1" cat= -> true }t
t{ -0E0 fnum+  s" .000E1" cat= -> true }t
[THEN]

t{ .1234e5 fnum+  s" .123E5" cat= -> true }t
t{ -.12e5  fnum+ s" -.120E5" cat= -> true }t

4 set-fnum-digs
t{ .1234e5 fnum+  s" .1234E5" cat= -> true }t
t{ -.12e5  fnum+ s" -.1200E5" cat= -> true }t

t{ .12346e0 fnum+ .5e1 fnum+ s" .1235E0.5000E1" cat= -> true }t

: tset-fnum-digs  ( u -- #extra.digs| )
  ['] set-fnum-digs CATCH
  IF ( u) MAX-FNUM-DIGS - THEN ;

\ test exception
t{ MAX-FNUM-DIGS 1-  set-fnum-digs -> }t
t{ MAX-FNUM-DIGS     set-fnum-digs -> }t
t{ MAX-FNUM-DIGS    tset-fnum-digs -> }t
t{ MAX-FNUM-DIGS 1+ tset-fnum-digs -> 1 }t

: tfnum+  ( f: num -- |-43 )
  ['] fnum+ CATCH ?dup IF >r fdrop r> THEN ;

4 set-fnum-digs
t{ -.12e5 tfnum+ s" -.1200E5" cat= -> true }t
t{ 0e 0e f/ tfnum+ -> -43 }t

[ELSE]
  COMMENT Floating point words not tested.
[THEN]

TESTING  DATE&TIME+

\ This will be wrong in the rare event that the execution time
\ interval overlaps a second boundary:

\ date&time+  ( -- )
t{ date&time+ date&time+ ENDCAT
  date&time+ ENDCAT $dup $+ $+ ENDCAT $s> $s> s= -> true }t

VERBOSE @ [IF]
blue-text
date&time+ ENDCAT $. cr
normal-text
[THEN]
?.errors GFORTH-HOST [IF] ?.cr [THEN]